function [ SongLabel, I_start_NoSong, I_end_NoSong, SineTrain, PulseTrain, bTrain, I_start_PulseSong, I_end_PulseSong, I_start_SineSong, I_end_SineSong, timelag] = SongTypeLabeling( data, pulses, sines , species)

%SinePulse_SwitchTimes_Function.m is a function which detects the times of switches from sine to pulse and gives them out.
%   Detailed explanation
%   INPUT-Arguements are the outputs of SongSegmenter.m (Arthur et al., 2013)
%   % data: a vector of song
%   % params: sampling frequency fs and other parameter of importance
%   % pulses: times? of pulses
%   % sines: times? of sine song
%   OUTPUT-Arguements
%   % SongLabel: vector of Labels (1 for Sine, -1 for Pulse and 0 for No Song)
%   % I_start_NoSong: Start Index of No Song
%   % I_stop_NoSong: Stop Index of No Song
%    Copyright 2015 Birgit Brggemeier

%% Define Constants
if strcmpi(species, 'melanogaster') == 1 
    quadruple_IPI = 4* 0.035; % melanogaster
elseif strcmpi(species, 'simulans') == 1 
    quadruple_IPI = 4* 0.045; % simulans (see e.g. Ritchie 1999 or Kyriacou 1989)
elseif strcmpi(species, 'yakuba') == 1 
    quadruple_IPI = 4* 0.124; % yakuba (see Demetriades 1999)
else
    error('SongTypeLabeling:SpeciesDescriptionWrong', 'Wrong species description. You can choose from "Yakuba", "Simulans" and "Melanogaster" only.');
end


timelag = floor(quadruple_IPI * data.fs); % 'timelag' is a parameter which defines the tolerated time lag between a sine and pulse train to be considered a SWITCH
% the timelag is in units of sampling, which are coded by data.fs

% I chose the constant of 0.08 as Arthur et al report a maximum IPI of 0.08
% seconds for Drosophila Melanogaster

%% Identification of PULSE TRAINS: converging of single Pulses to Pulse Train on the basis of timelag

pTrainNum = 1;

for ii = 1: length(pulses.AmpCull.w0)
    
    if ii ==1
        
        pTrain(ii) = pTrainNum;
        
    elseif ii >1 && ii < length(pulses.AmpCull.w0)
        
        if pulses.AmpCull.w0(ii+1) <= pulses.AmpCull.w1(ii) + timelag; % if the onset of a pulse is less than 80ms distant from the end of the previous pulse, they are considered to belong to one pulse train
            
            pTrain(ii) = pTrainNum;
            
        elseif pulses.AmpCull.w0(ii+1) > pulses.AmpCull.w1(ii) + timelag;
            
            pTrain(ii) = pTrainNum;
            
            pTrainNum = pTrainNum + 1;
            
            pTrain(ii + 1) = pTrainNum;
            
        end
        
    elseif ii == length(pulses.AmpCull.w0);
        
        pTrain(ii) = pTrainNum;
        
    end
end

%% Generate two vectors: Start and End of PULSE TRAIN

for ii = 1: length(unique(pTrain));
    
    IndexPulse.start(ii) = find(pTrain==ii,1,'first'); % indices for pulses.AmpCull, NOT sample/time itself
    IndexPulse.stop(ii) = find(pTrain==ii,1,'last');
    
end

PulseTrain.start = pulses.AmpCull.w0(IndexPulse.start);
PulseTrain.stop = pulses.AmpCull.w1(IndexPulse.stop);


%% Identification of SINE TRAINS: conversion of short chopped Sine Trains which are not farther from one another than timelag


sTrainNum = 1;


if isempty(sines.LengthCull.start) == 1
   sTrain = 0; 
   IndexSine.start = 0;
   IndexSine.stop = 0;
   SineTrain.start = 0;
   SineTrain.stop = 0;
else
for ii = 1: length(sines.LengthCull.start)
    
    if ii==1
        sTrain(ii) = sTrainNum;
    elseif ii >1 && ii<length(sines.LengthCull.start)
        
        if sines.LengthCull.start(ii+1) <= sines.LengthCull.stop(ii) + timelag; % if the onset of a pulse is less than 80ms far from the end of the previous sine train, they are considered to belong to one sine train
            
            sTrain(ii) = sTrainNum;
            
        elseif sines.LengthCull.start(ii+1) > sines.LengthCull.stop(ii) + timelag;
            
            sTrain(ii) = sTrainNum;
            
            sTrainNum = sTrainNum + 1;
            
            sTrain(ii + 1) = sTrainNum;
            
        end
        
    elseif ii == length(sines.LengthCull.start);
        
        sTrain(ii) = sTrainNum;
        
    end
end

%% Generate two vectors: Start and End of SINE TRAIN

for ii = 1: length(unique(sTrain));
    IndexSine.start(ii) = find(sTrain==ii,1,'first'); % indices for pulses.AmpCull, NOT sample/time itself
    IndexSine.stop(ii) = find(sTrain==ii,1,'first');
end

% exclude SineTrains with a maximum Amplitude below the costant minSineAmp

minSineAmp = 0;%0.0005;

for ii = 1: length(IndexSine.start);
    if max(data.d(sines.LengthCull.start(IndexSine.start(ii)):sines.LengthCull.stop(IndexSine.stop(ii)))) >= minSineAmp;
        
        IndexSine.start(ii) = IndexSine.start(ii);
        IndexSine.stop(ii) = IndexSine.stop(ii);
    else
        IndexSine.start(ii) = 0;
        IndexSine.stop(ii) = 0;
    end
end

IndexSine.start(IndexSine.start == 0) = [];
IndexSine.stop(IndexSine.stop == 0) = [];

SineTrain.start = sines.LengthCull.start(IndexSine.start);
SineTrain.stop = sines.LengthCull.stop(IndexSine.stop);
end

%% Identification of 'OVERLAP' between Sine, Pulse and timelag

% Generation of 0-1 vectors indicating SINE respectively PULSE trains
Vsine = zeros(1,length(data.d));
Vpulse = zeros(1,length(data.d));

if isempty(sines.LengthCull.start) == 1
    
else
indexF = @(a,b) a:b;
Isine = arrayfun(indexF, SineTrain.start, SineTrain.stop,'UniformOutput',false);
Isine = cat(2,Isine{:});
Ipulse = arrayfun(indexF, PulseTrain.start, PulseTrain.stop,'UniformOutput',false);
Ipulse = cat(2,Ipulse{:});

Vsine(Isine) = 1;
Vpulse(Ipulse) = 1;

i_S_P = zeros(1,length(SineTrain.stop));
i_P_S = zeros(1,length(PulseTrain.stop));

% Switch from Sine to Pulse
for ii = 1: length(SineTrain.stop)
    
    i_S_P(ii) = SineTrain.stop(ii);
    
    if  i_S_P(ii)+1+timelag < length(data.d) && sum(Vpulse(i_S_P(ii)+1: i_S_P(ii)+1+ timelag)) ~= 0; % a switch between sine and pulse is considered to have occured if pulse song follows in 1 second after sine
        
        hS{ii} = find(Vpulse(i_S_P(ii)+1: i_S_P(ii)+1+timelag) == 1);
        %assign Sine-Label to 'No-Song' in buffer time
        Vsine(i_S_P(ii)+1:i_S_P(ii)+hS{ii}(1)) = 1;
        
    end
end




% Switch from Pulse to Sine
for ii = 1: length(PulseTrain.stop)
    
    i_P_S(ii) = PulseTrain.stop(ii);
    
    if  i_P_S(ii)+1+timelag < length(data.d) && sum(Vsine(i_P_S(ii)+1: i_P_S(ii)+1+ timelag)) ~= 0; % a switch between sine and pulse is considered to have occured if pulse song follows in 1 second after sine
        
        hP{ii} = find(Vsine(i_P_S(ii)+1: i_P_S(ii)+1+timelag) == 1);
        %assign Pulse-Label to 'No-Song' in buffer time
        Vpulse(i_P_S(ii)+1:i_P_S(ii)+hP{ii}(1)) = 1;
        
    end
end
end



%% Identify No-Song End-Indices

% Initialise and construct Variable SongLabel

SongLabel = zeros(1, length(data.d));

VsineL = logical(Vsine);
VpulseL = logical(Vpulse);

SongLabel(VpulseL) = -1; % Label for Pulse = -1
SongLabel(VsineL) = 1;   % Label for Sine = 1

%% Label for No-Song = 0

I_NoSong = find(SongLabel == 0);
I_NoSong = [-1 I_NoSong];

% Identify Start Indices of NoSong
h_No_1 = find(diff(I_NoSong) > 1);
I_start_NoSong = I_NoSong(h_No_1+1);

% Identify End Indices of NoSong
I_NoSong = [I_NoSong I_NoSong(end)+2];

h_No_2 = find(diff(I_NoSong(2:end)) > 1);
I_end_NoSong = I_NoSong(h_No_2+1);

%% Label for Sine-Song = 1

I_SineSong = find(SongLabel == 1);
I_SineSong = [-1 I_SineSong];

% Identify Start Indices of NoSong
h_Sine_1 = find(diff(I_SineSong) > 1);
I_start_SineSong = I_SineSong(h_Sine_1+1);

% Identify End Indices of NoSong
I_SineSong = [I_SineSong I_SineSong(end)+2];

h_Sine_2 = find(diff(I_SineSong(2:end)) > 1);
I_end_SineSong = I_SineSong(h_Sine_2+1);

%% Label for Pulse-Song = -1

I_PulseSong = find(SongLabel == -1);
I_PulseSong = [-1 I_PulseSong];

% Identify Start Indices of NoSong
h_Pulse_1 = find(diff(I_PulseSong) > 1);
I_start_PulseSong = I_PulseSong(h_Pulse_1+1);

% Identify End Indices of NoSong
I_PulseSong = [I_PulseSong I_PulseSong(end)+2];

h_Pulse_2 = find(diff(I_PulseSong(2:end)) > 1);
I_end_PulseSong = I_PulseSong(h_Pulse_2+1);


%% Identify Song Bout Trains

I_start_Song = I_end_NoSong + 1;

I_start_Song(I_start_Song > length(data.d)) = [];

I_end_Song = I_start_NoSong - 1;

I_end_Song(I_end_Song < 1) = [];


bTrainNum = 1;

bTrain = zeros(1,length(data.d));

for ii = 1: length(I_start_Song);
    
    bTrain(I_start_Song(ii):I_end_Song(ii)) = bTrainNum;
    bTrainNum = bTrainNum + 1;
    
end

end

